# -*- coding: utf-8 -*-

import socket
import struct
import sys
import binascii

# STUN服务器的地址和端口
STUN_SERVER = ("stun.miwifi.com", 3478)

# STUN消息类型和魔法Cookie
STUN_BINDING_REQUEST = 0x0001
MAGIC_COOKIE = 0x2112A442

def create_stun_request():
    """创建STUN请求包"""
    transaction_id = b"".join([struct.pack("B", i) for i in range(12)])  # 事务ID（12字节）
    stun_request = struct.pack("!HHI12s", STUN_BINDING_REQUEST, 0, MAGIC_COOKIE, transaction_id)
    return stun_request, transaction_id

def parse_stun_response(data, transaction_id):
    """解析STUN响应包"""
    try:
        print("解析STUN响应数据（十六进制）：{}".format(binascii.hexlify(data)))
        if len(data) < 20:
            print("STUN响应数据长度不足（长度:{}），无法解析".format(len(data)))
            return None
        # 解析响应头
        response_type, length, magic_cookie, received_transaction_id = struct.unpack("!HHI12s", data[:20])
        if magic_cookie != MAGIC_COOKIE:
            print("响应的Magic Cookie不匹配")
            return None
        if received_transaction_id != transaction_id:
            print("响应的事务ID不匹配")
            return None

        # 提取STUN属性
        attributes = data[20:]
        while len(attributes) >= 4:
            attr_type, attr_length = struct.unpack("!HH", attributes[:4])
            if len(attributes) < 4 + attr_length:
                print("属性长度不足，无法继续解析")
                break
            attr_value = attributes[4:4 + attr_length]
            if attr_type == 0x0020:  # XOR-MAPPED-ADDRESS属性
                return parse_xor_mapped_address(attr_value, magic_cookie, transaction_id)
            attributes = attributes[4 + attr_length:]
        print("未找到有效的地址映射属性")
        return None
    except Exception as e:
        print("解析STUN响应时发生错误：{}".format(e))
        return None

def parse_xor_mapped_address(attr_value, magic_cookie, transaction_id):
    """解析 XOR-MAPPED-ADDRESS 属性"""
    try:
        # 检查属性长度
        if len(attr_value) < 8:
            print("XOR-MAPPED-ADDRESS属性长度不足")
            return None

        # 解析地址族和端口
        family, xor_port = struct.unpack("!BH", attr_value[:3])
        if family == 0x01:  # IPv4
            port = xor_port ^ (magic_cookie >> 16)  # 解密端口
            ip = struct.unpack("!I", attr_value[4:8])[0] ^ magic_cookie  # 解密IP
            ip = socket.inet_ntoa(struct.pack("!I", ip))
            return ip, port
        elif family == 0x02:  # IPv6
            print("暂不支持IPv6地址")
            return None
        else:
            print("未知地址族：{}，可能是解析错误".format(family))
            return None
    except Exception as e:
        print("解析XOR-MAPPED-ADDRESS时发生错误：{}".format(e))
        return None

def determine_nat_type(interface_ip):
    """判断指定网口的NAT类型"""
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.settimeout(2)
        sock.bind((interface_ip, 0))

        stun_request, transaction_id = create_stun_request()
        sock.sendto(stun_request, STUN_SERVER)

        print("等待STUN服务器响应...")
        data, server = sock.recvfrom(1024)
        print("接收到STUN响应数据长度：{} 字节".format(len(data)))
        public_ip, public_port = parse_stun_response(data, transaction_id)
        if not public_ip:
            print("无法解析STUN响应")
            return

        local_ip, local_port = sock.getsockname()
        print("本地地址：{}:{}".format(local_ip, local_port))
        if (public_ip, public_port) == (local_ip, local_port):
            print("NAT类型：开放（Open NAT）")
        else:
            print("NAT类型：受限（可能是Symmetric NAT或Restricted NAT），外部IP：{}, 外部端口：{}".format(public_ip, public_port))

    except socket.timeout:
        print("STUN服务器未响应，无法判断NAT类型")
    except Exception as e:
        print("发生错误：{}".format(e))
    finally:
        sock.close()

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("用法: python nat_type_checker.py <网口IP地址>")
        sys.exit(1)

    interface_ip = sys.argv[1]
    determine_nat_type(interface_ip)

